@@ -161,6 +161,8 @@ func Parse(s *types.Schema, schemaString string, useStringDescriptions bool) err
 		}
 	}
 
+	s.SchemaString = schemaString
+
 	return nil
 }
 
@@ -408,19 +410,19 @@ func parseObjectDef(l *common.Lexer) *types.ObjectTypeDefinition {
 			continue
 		}
 
-		if l.Peek() == scanner.Ident {
-			l.ConsumeKeyword("implements")
-
-			for l.Peek() != '{' && l.Peek() != '@' {
-				if l.Peek() == '&' {
-					l.ConsumeToken('&')
-				}
-
-				object.InterfaceNames = append(object.InterfaceNames, l.ConsumeIdent())
-			}
-			continue
+		if l.Peek() != scanner.Ident {
+			break
 		}
 
+		l.ConsumeKeyword("implements")
+
+		for l.Peek() != '{' && l.Peek() != '@' {
+			if l.Peek() == '&' {
+				l.ConsumeToken('&')
+			}
+
+			object.InterfaceNames = append(object.InterfaceNames, l.ConsumeIdent())
+		}
 	}
 	l.ConsumeToken('{')
 	object.Fields = parseFieldsDef(l)
@@ -511,10 +513,21 @@ func parseDirectiveDef(l *common.Lexer) *types.DirectiveDefinition {
 		l.ConsumeToken(')')
 	}
 
-	l.ConsumeKeyword("on")
+	switch x := l.ConsumeIdent(); x {
+	case "on":
+		// no-op; Go doesn't fallthrough by default
+	case "repeatable":
+		d.Repeatable = true
+		l.ConsumeKeyword("on")
+	default:
+		l.SyntaxError(fmt.Sprintf(`unexpected %q, expecting "on" or "repeatable"`, x))
+	}
 
 	for {
 		loc := l.ConsumeIdent()
+		if _, ok := legalDirectiveLocationNames[loc]; !ok {
+			l.SyntaxError(fmt.Sprintf("%q is not a legal directive location (options: %v)", loc, legalDirectiveLocationNames))
+		}
 		d.Locations = append(d.Locations, loc)
 		if l.Peek() != '|' {
 			break
@@ -584,3 +597,25 @@ func parseFieldsDef(l *common.Lexer) types.FieldsDefinition {
 	}
 	return fields
 }
+
+var legalDirectiveLocationNames = map[string]struct{}{
+	"SCHEMA":                 {},
+	"SCALAR":                 {},
+	"OBJECT":                 {},
+	"FIELD_DEFINITION":       {},
+	"ARGUMENT_DEFINITION":    {},
+	"INTERFACE":              {},
+	"UNION":                  {},
+	"ENUM":                   {},
+	"ENUM_VALUE":             {},
+	"INPUT_OBJECT":           {},
+	"INPUT_FIELD_DEFINITION": {},
+	"QUERY":                  {},
+	"MUTATION":               {},
+	"SUBSCRIPTION":           {},
+	"FIELD":                  {},
+	"FRAGMENT_DEFINITION":    {},
+	"FRAGMENT_SPREAD":        {},
+	"INLINE_FRAGMENT":        {},
+	"VARIABLE_DEFINITION":    {},
+}
